/*
 Prototypes for MathPad's callback routines.
 
 This header file is for XFuns that use A4 globals (or for CFuns).
 The file "callbackg.c" implements these routines by using a
 global funptr "callback" which must be set on initialzation.

 The compiler must be set so that type "double" is a SANE 80-bit.
 
*/

typedef short (*funptr) ();
typedef struct expr	*EXPR;	/* internal details aren't needed by XFun */

extern funptr callback;

#ifdef __cplusplus
extern "C" {
#endif

/* ---------- control routines ---------------------------- */

extern void AddXfun(char *name,char *parms,funptr entry,funptr predef);
  /* Install a new function from a code resource. */
  //(internally defined a static)
  
extern void AddFunDim(char *name, long lim);
  /* Setup routine to allow the XFun to return values based on the array index.
   This routine must be called by the predef routine. It can be called multiple times
   to set up a multidimensional return value. Array index values are appended to the
   parameter list */

extern void ErrMsg(char *fmt,char *str);
  /* Show an error message in the status line.
    fmt is a printf() format string. str is an optional parameter
    Sets stop flag to tell MathPad to stop all evaluation. */

extern short Stopped(void);
  /* Check value of stop flag to see if evaluation should be stopped. */

extern void SpinWatch(void);
  /* Allow a computation to be switched to background or stopped via command period.
   Stopped() must be checked to see if command period was hit.
   SpinWatch() should be called at least twice per second.
   It can be called at a much higher rate without much loss. */

extern void ReturnVoid(void);
  /* Allow the XFun to return false without generating an error message. */


/* ------------- function parameter access ---------------- */
/* Parameters are numbered from right to left starting at 0 */

extern short GetParmVal(long n,double *num);
  /* Get the value of a simple numeric parameter.
   Returns false if the value is an array or undefined */

extern void MakeParmExpr(long n,EXPR *xpr);
  /* Make a parameter expression. (Used to access arrays).
   The expression must be deallocated with FreeExpr(xpr). */

extern short GetParmName(long n,char **name);
  /* Get a name parameter. (Used to get a filename or other string).
    Returns false if the parameter was an expression or defined variable */



/* ------------------ global variable access ---------------- */

extern short GetVarVal(char *name,double *num);
  /* Evaluate the named global variable.
    Returns false if the value is an array or undefined */

extern void GetVarString(char *name,char **str);
  /* Returns the string value of a named global variable.
    Returns an empty string if unless the variable was set to a string */

extern void MakeVarExpr(char *name,EXPR *xpr);
  /* Make a global variable expression. (Used to access arrays).
   The expression must be deallocated with FreeExpr(xpr). */

extern short SetVarVal(char *name,double num);
  /* Assign a single value to the named global variable.
     Returns false if the variable is an illegal destination. */

extern short SetVarMatrix(char *name,double *arr,long rows,long cols);
  /* Assign a 1D or 2D array to the named global variable. 
     Returns false if the variable is an illegal destination.
     The array must be allocated via NewPtr() and will be deallocated by MathPad. */

extern void FoldVar(char *name,long lim);
  /* Add a dimension to the variable set by SetVarMatrix(). Can be called multiple
     times to create more dimensions. */



/* ------------ general expression routines ----------------- */

extern short GetExprMatrix(EXPR xpr,double **mat,long *rows,long *cols);
  /* Allocate memory and evaluate all elements of a 1D or 2D matrix expression into memory.
   Returns a pointer to the matrix in mat.
   A scalar will return cols=0 and rows=0. A 1D array will return cols=0.
   An array with more than 2 dimensions will generate an error message and return false.
   The matrix allocated by GetExprMatrix() should be deallocated with DisposPtr() */

extern short ProbeExpr(EXPR xpr,double *num,short *isarray,long *count);
  /* Find out if an expression is a scalar or an array.
   If scalar, returns true and sets num.
   If array, returns false and sets isarray and count. A count of 0 means infinite array.
   If the return value and isarray are both false the expression was undefined. */

extern void AddIndex(EXPR *xpr,double **iptr);
  /* Add an index to allow evaluating individual array elements.
   The double at iptr can be modified to index through the array.
   To access a single element, an index must be added for each dimension of the array. */

extern void RemoveIndex(EXPR *xpr);
  /* Can be used to remove the last index added by AddIndex(). Normally FreeExpr() is
    be used to free the entire expression so this call is not required. */

extern short EvalExpr(EXPR xpr,double *num);
  /* Evaluate an expression. Returns false if xpr is an array or undefined.*/   

extern void FreeExpr(EXPR xpr);
  /* Free memory allocated to an expression.
   Use to deallocate from MakeParmExpr() or MakeVarExpr() */


/* ---------- PICT overlay ---------------- */

extern void SetPlotPICT(PicHandle thePic);
  /* Allows an XFun to overlay PICT graphics on the plot. Overlay goes on current strip.
   The XFun must allocate and create thePic. MathPad will dispose of it.  */

extern void SizePlotPICT(double left,double bot,double width,double ht);
  /* Specify where to draw the PICT in data coordinates.
    Default is to size the PICT to fill the data plot area.  */


/* ---------- Document info ----------- */

short GetDocInfo(long *docref,FSSpec *fspec,WindowPtr *plotwindow);
  /* Get info about the document being evaluated. Returns false if no open doc. */

extern void GetAngleUnits(double *unit);
  /* Get the currently selected multiplier trig function parameters.
     1.0 for radians, .017 for degrees */

extern void GetAxisRect(Rect *axisrect);
  /* Get the current data axis rectangle in plot window coordinates */

extern void GetAxisLimits(double limits[6]);
  /* Get the the current values of xmin,xmax,ymin,ymax,zmin,zmax */
  
extern void GetPlotInfo(double info[12]);
  /* Get the the current values of xlo,xhi,ylo,yhi,zlo,zhi,
     traceclick,xclick,yclick,zclick,ixclick,iyclick */


/* --------------- Event handlers -------------*/

extern void InstallEventHandler(funptr handler);
  /* install external event handler */
  
extern void SetCursorRegion(Cursor *curs,RgnHandle inside);
  /* set region for mouse moved events */

extern void DoMenuItem(long selector);
  /* Perform the action for a MathPad menu item. This allows the XFun event handler
     intercept info from MenuSelect(theEvent->where) and pass back selections */


/* -------------- Utillity --------------------*/
extern short Sprintf(char *buf,char *fmt,double num);
  /* Allows XFun some access to sprintf() without linking its own ANSI library */

extern void MapData(double x,double y,short *xpixel,short *ypixel);
  /* use current plot window scaling to convert data coords to pixel coords */



/* -------------- CFun only -------------------*/
extern void AddCFun(char *name,char *parms,funptr entry,funptr predef);
  /* install external fun from code frag */

void AddLoopIndex(double **iptr);
  /* Add global index to be used with subsequent EvalExpr() calls */
  
void RemoveLoopIndex(void);
  /* Remove global index. Must be called to balance each AddLoopIndex() call */


#ifdef __cplusplus
}
#endif
